@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2022 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -10,6 +10,7 @@
 
 import functools
 
+from traits.observation._anytrait_filter import anytrait_filter
 from traits.observation._dict_item_observer import DictItemObserver
 from traits.observation._filtered_trait_observer import FilteredTraitObserver
 from traits.observation._list_item_observer import ListItemObserver
@@ -18,8 +19,12 @@ from traits.observation._named_trait_observer import NamedTraitObserver
 from traits.observation._observer_graph import ObserverGraph
 from traits.observation._set_item_observer import SetItemObserver
 
-# ObserverExpression is a public user interface for constructing ObserverGraph.
 
+#: Maximum number of parsed observer expressions stored in the LRU caches
+_OBSERVER_EXPRESSION_CACHE_MAXSIZE = 256
+
+
+# ObserverExpression is a public user interface for constructing ObserverGraph.
 
 class ObserverExpression:
     """
@@ -31,17 +36,7 @@ class ObserverExpression:
     functions provided in this module, e.g. ``trait``.
     """
 
-    def __eq__(self, other):
-        """ Return true if the other value is an ObserverExpression with
-        equivalent content.
-
-        Returns
-        -------
-        bool
-        """
-        if type(other) is not type(self):
-            return False
-        return self._as_graphs() == other._as_graphs()
+    __slots__ = ()
 
     def __or__(self, expression):
         """ Create a new expression that matches this expression OR
@@ -101,6 +96,23 @@ class ObserverExpression:
         """
         return self.then(match(filter=filter, notify=notify))
 
+    def anytrait(self, notify=True):
+        """ Create a new expression for observing all traits.
+
+        Events emitted (if any) will be instances of
+        :class:`~traits.observation.events.TraitChangeEvent`.
+
+        Parameters
+        ----------
+        notify : bool, optional
+            Whether to notify for changes. Default is to notify.
+
+        Returns
+        -------
+        new_expression : ObserverExpression
+        """
+        return self.match(filter=anytrait_filter, notify=notify)
+
     def metadata(self, metadata_name, notify=True):
         """ Return a new expression for observing traits where the given
         metadata is not None.
@@ -260,12 +272,23 @@ class SingleObserverExpression(ObserverExpression):
     """ Container of ObserverExpression for wrapping a single observer.
     """
 
+    __slots__ = ("_observer",)
+
     def __init__(self, observer):
-        self.observer = observer
+        self._observer = observer
+
+    def __hash__(self):
+        return hash((type(self).__name__, self._observer))
+
+    def __eq__(self, other):
+        return (
+            type(self) is type(other)
+            and self._observer == other._observer
+        )
 
     def _create_graphs(self, branches):
         return [
-            ObserverGraph(node=self.observer, children=branches),
+            ObserverGraph(node=self._observer, children=branches),
         ]
 
 
@@ -280,10 +303,22 @@ class SeriesObserverExpression(ObserverExpression):
         Right expression to be joined in series.
     """
 
+    __slots__ = ("_first", "_second")
+
     def __init__(self, first, second):
         self._first = first
         self._second = second
 
+    def __hash__(self):
+        return hash((type(self).__name__, self._first, self._second))
+
+    def __eq__(self, other):
+        return (
+            type(self) is type(other)
+            and self._first == other._first
+            and self._second == other._second
+        )
+
     def _create_graphs(self, branches):
         branches = self._second._create_graphs(branches=branches)
         return self._first._create_graphs(branches=branches)
@@ -300,10 +335,22 @@ class ParallelObserverExpression(ObserverExpression):
         Right expression to be joined in parallel.
     """
 
+    __slots__ = ("_left", "_right")
+
     def __init__(self, left, right):
         self._left = left
         self._right = right
 
+    def __hash__(self):
+        return hash((type(self).__name__, self._left, self._right))
+
+    def __eq__(self, other):
+        return (
+            type(self) is type(other)
+            and self._left == other._left
+            and self._right == other._right
+        )
+
     def _create_graphs(self, branches):
         left_graphs = self._left._create_graphs(branches=branches)
         right_graphs = self._right._create_graphs(branches=branches)
@@ -411,6 +458,21 @@ def match(filter, notify=True):
     return SingleObserverExpression(observer)
 
 
+def anytrait(notify=True):
+    """ Create a new expression for observing all traits on an object.
+
+    Events emitted (if any) will be instances of
+    :class:`~traits.observation.events.TraitChangeEvent`.
+
+    Parameters
+    ----------
+    notify : bool, optional
+        Whether to notify for changes.
+    """
+    observer = FilteredTraitObserver(notify=notify, filter=anytrait_filter)
+    return SingleObserverExpression(observer)
+
+
 def metadata(metadata_name, notify=True):
     """ Return a new expression for observing traits where the given metadata
     is not None.
@@ -486,3 +548,18 @@ def trait(name, notify=True, optional=False):
     observer = NamedTraitObserver(
         name=name, notify=notify, optional=optional)
     return SingleObserverExpression(observer)
+
+
+@functools.lru_cache(maxsize=_OBSERVER_EXPRESSION_CACHE_MAXSIZE)
+def compile_expr(expr):
+    """ Compile an ObserverExpression to a list of ObserverGraphs.
+
+    Parameters
+    ----------
+    expr : ObserverExpression
+
+    Returns
+    -------
+    list of ObserverGraph
+    """
+    return expr._as_graphs()
